"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "SchedulerNode", {
    enumerable: true,
    get: ()=>SchedulerNode
});
const _aliceBob = require("alice-bob");
const _eventToolkit = require("event-toolkit");
const _syncedSet = require("synced-set");
const _schedulerCore = require("./scheduler-core");
const _schedulerEvent = require("./scheduler-event");
let SchedulerNode = class SchedulerNode extends EventTarget {
    static registeredCtx = new WeakSet();
    static async register(context) {
        if (this.registeredCtx.has(context)) return;
        // @ts-ignore
        await context.audioWorklet.addModule(new URL('./scheduler-processor.js', require("url").pathToFileURL(__filename).toString()).href);
        this.registeredCtx.add(context);
    }
    static async create(context) {
        await this.register(context);
        return new this(context);
    }
    node;
    worklet;
    targetNodes;
    eventGroups;
    createEventGroup() {
        const eventGroup = new _schedulerEvent.SchedulerEventGroup();
        this.eventGroups.add(eventGroup);
        return eventGroup;
    }
    removeEventGroup(eventGroup) {
        this.eventGroups.delete(eventGroup);
    }
    constructor(context){
        super();
        this.context = context;
        this.targetNodes = new Set();
        this.eventGroups = new _syncedSet.SyncedSet({
            send: _eventToolkit.queue.task((payload, cb)=>{
                //!? 'sending'
                this.node.port.postMessage({
                    eventGroups: _schedulerCore.core.serialize(payload)
                });
                cb();
            }),
            pick: [],
            reducer: (eventGroup)=>({
                    targets: eventGroup.targets,
                    events: eventGroup.events
                }),
            equal: (prev, next)=>prev.targets === next.targets && prev.events === next.events
        });
        this.node = new AudioWorkletNode(this.context, 'scheduler');
        const [scheduler, worklet] = new _aliceBob.Alice((data)=>void this.node.port.postMessage({
                rpc: data
            })).agents({
            debug: false
        });
        this.node.port.onmessage = ({ data  })=>{
            if (data.rpc) scheduler.receive(data.rpc);
        };
        // TODO: handle onprocessorerror
        this.worklet = worklet;
    }
    start() {
        return this.worklet.start();
    }
    stop() {
        this.worklet.stop();
    }
    setBpm(bpm) {
        return this.worklet.setBpm(bpm);
    }
    connect(targetNode) {
        if (!this.targetNodes.has(targetNode)) {
            this.node.connect(targetNode, 0, targetNode.numberOfInputs - 1);
        }
        return targetNode;
    }
    disconnect(targetNode) {
        if (this.targetNodes.has(targetNode)) {
            this.node.disconnect(targetNode, 0, targetNode.numberOfInputs - 1);
        }
    }
    context;
};

//# sourceMappingURL=scheduler-node.js.map