"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "getEventsInRange", {
    enumerable: true,
    get: ()=>getEventsInRange
});
const getEventsInRange = (events, loop, loopStart, loopEnd, currentTime, sampleTime, quantumDurationTime, playbackStartTime)=>{
    let startTime;
    let endTime;
    let offsetTime = 0;
    let loopDuration = 0;
    const results = [];
    if (loop) {
        loopDuration = loopEnd - loopStart;
        offsetTime = currentTime - playbackStartTime - loopStart;
        if (offsetTime < 0) {
            return results;
        }
        startTime = offsetTime % loopDuration;
        endTime = (startTime + quantumDurationTime) % loopDuration;
        if (startTime < sampleTime) startTime = 0;
        if (endTime < sampleTime) endTime = 0;
    } else {
        startTime = currentTime - playbackStartTime;
        endTime = startTime + quantumDurationTime;
    }
    for (const event of events){
        const eventTime = event.midiEvent.receivedTime * 0.001 // ms to seconds
        ;
        const isPast = endTime < startTime;
        if (eventTime === startTime && startTime === endTime || (isPast ? eventTime >= loopStart && eventTime < endTime : eventTime >= startTime && eventTime < endTime)) {
            let receivedTime;
            if (loop) {
                receivedTime = eventTime + Math[isPast ? 'ceil' : 'floor'](startTime / loopDuration) * loopDuration + playbackStartTime + (offsetTime - startTime);
            } else {
                // TODO: -startTime ?
                receivedTime = eventTime + playbackStartTime;
            }
            receivedTime *= 1000;
            results.push([
                receivedTime,
                event
            ]);
        }
    }
    return results;
};

//# sourceMappingURL=util.js.map